# -*- coding: utf-8 -*-
from acomea.utenti.models import AcomeaProfile
from django import forms
from django.contrib import admin
from django.contrib.auth.models import User
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.forms import UserChangeForm, UserCreationForm
from django.core.urlresolvers import reverse
from django.conf.urls import patterns, url
from django.utils.translation import ugettext_lazy as _

from jmb.core.admin.options import ExtendibleModelAdmin

from .admin_views import *
from .models import LeadProfile, ClienteProfile, PromotoreProfile, UtenteReteProfile, Rete, \
    RichiestaAttivazioneCliente, AcomeaProfiles, AcomeaRegistrationProfile


class LeadProfileAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_user', 'abilitato',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon',
    )


class ClienteProfileAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_user', 'abilitato',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon',
    )


class PromotoreProfileAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_user', 'abilitato',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon',
    )


class UtenteReteProfileAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_user', 'abilitato',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon',
    )
    fieldsets = (
        (None, {'fields': (
            'abilitato', 'rete',
        ), }),
    )


class ReteAdmin(ExtendibleModelAdmin):
    list_display = (
        'nome',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon',
    )


class RichiestaAttivazioneClienteAdmin(ExtendibleModelAdmin):
    list_display = (
        'leadprofile',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon',
    )


admin.site.register(AcomeaProfiles)
admin.site.register(LeadProfile, LeadProfileAdmin)
admin.site.register(ClienteProfile, ClienteProfileAdmin)
admin.site.register(PromotoreProfile, PromotoreProfileAdmin)
admin.site.register(UtenteReteProfile, UtenteReteProfileAdmin)
admin.site.register(Rete, ReteAdmin)
admin.site.register(RichiestaAttivazioneCliente, RichiestaAttivazioneClienteAdmin)


class AcomeAUserSearchForm(forms.ModelForm):
    class Meta:
        model = User
        display_fields = (
            'email__exact', 'first_name__icontains', 'last_name__icontains'
        )

    def __init__(self, *args, **kws):
        super(AcomeAUserSearchForm, self).__init__(*args, **kws)
        self.fields['email__exact'] = forms.CharField(label=_("Email"), required=False)
        self.fields['first_name__icontains'] = forms.CharField(label=_("Nome"), required=False)
        self.fields['last_name__icontains'] = forms.CharField(label=_("Cognome"), required=False)


class AcomeAUserChangeForm(UserChangeForm):

    error_messages = {
            'duplicate_email': _("Un utente con questa email esiste gia'"),
        }
    def __init__(self, *args, **kwargs):
        super(AcomeAUserChangeForm, self).__init__(*args, **kwargs)
        self.fields.pop('username',None)
        self.fields['username'] = forms.RegexField(
                                    label=_("Username"), max_length=50, regex=r"^[\w.@+-]+$",
                                    help_text=_("Required. 30 characters or fewer. Letters, "
                                                "digits and @/./+/-/_ only.")
                                  )

    def clean_email(self):
        ## controllo unicita' email
        email = self.cleaned_data["email"]
        if User.objects.filter(email=email).exclude(pk=self.instance.pk).count()>0:
            raise forms.ValidationError(self.error_messages['duplicate_email'])
        return email


class AcomeAUserCreationForm(UserCreationForm):

    def __init__(self, *args, **kwargs):
        super(AcomeAUserCreationForm, self).__init__(*args, **kwargs)
        self.error_messages.update({
            'duplicate_email': _("Un utente con questa email esiste gia'"),
        })
        self.fields.pop('username',None)
        self.fields['username'] = forms.RegexField(
                                    label=_("Username"), max_length=50, regex=r"^[\w.@+-]+$",
                                    help_text=_("Required. 30 characters or fewer. Letters, "
                                                "digits and @/./+/-/_ only.")
                                  )

    def clean_email(self):
        ## controllo unicita' email
        email = self.cleaned_data["email"]
        try:
            User.objects.get(email=email)
        except User.DoesNotExist:
            return email
        raise forms.ValidationError(self.error_messages['duplicate_email'])


class AcomeAUserAdmin(UserAdmin, ExtendibleModelAdmin):

    # readonly_fields = ('username',)

    list_display = (
        'id', 'username', 'first_name', 'last_name', 'get_edit_icon',
        'get_offusca_dati_link', 'get_simulazioneptf_link'
    )

    def get_simulazioneptf_link(self, obj):
        """
        Returns page simulazione ptf
        """
        return "<a href='%s' title='simulazione ptf' alt='simulazione ptf' target='_blank'>x</a>" % (
            reverse('admin:simulazione_ptf', args=[obj.id])
        )
    get_simulazioneptf_link.allow_tags = True
    get_simulazioneptf_link.short_description = 'SIM'

    def get_offusca_dati_link(self, obj):
        """
        Returns offusca dati
        """
        return "<a href='{0}' title='{1}' alt='{1}' onclick='return confirm({2})'>" \
            "<img src='/static/jmb/images/del.gif' alt='{1}' title='{1}'></a>".format(
                reverse('admin:offusca_dati', args=[obj.id]),
                'offusca dati',
                '"Sei sicuro? Le informazioni non potranno essere recuperate."'
            )
    get_offusca_dati_link.allow_tags = True
    get_offusca_dati_link.short_description = 'offusca'

    class Media:
        css = {"all": ("admin/css/acomeauser.css",)}

    # search_form = AcomeAUserSearchForm

    form = AcomeAUserChangeForm
    add_form = AcomeAUserCreationForm

    advanced_search_fields = (
        ('first_name__icontains:nome', 'last_name__icontains:cognome',),
        ('email__icontains:email',),
        ('is_active:utente attivo',)
    )

    actions = ['make_obfuscate']

    def get_actions(self, request):
        actions = super(AcomeAUserAdmin, self).get_actions(request)
        del actions['delete_selected']
        return actions

    def make_obfuscate(self, request, queryset):
        totals = queryset.count()
        obfuscated = 0
        error = 0

        for user in queryset:
            try:
                user.delete()
                obfuscated += 1
                self.log_change(request, user, _("Offuscato"))
            except Exception, e:
                error += 1
                messages.error(request, _('Error: {} - {}').format(user.email, e))
        messages.error(request, _("Total errors: {}").format(error))
        messages.info(request, _("Utenti offuscati: {} di {}").format(obfuscated, totals))
    make_obfuscate.short_description = _("Offusca utenti selezionati")

    def get_urls(self):
        urls = super(AcomeAUserAdmin, self).get_urls()
        my_urls = patterns(
            '',
            # lead
            url(
                r'^dashboard_lead/(?P<user_id>\d+)/$',
                self.admin_site.admin_view(dashboard_lead),
                name='dashboard_lead'),
            url(
                r'delete_lead/(?P<user_id>\d+)/$',
                self.admin_site.admin_view(delete_lead),
                name='delete_lead'),
            url(
                r'switch_lead/(?P<lead_id>\d+)/$',
                self.admin_site.admin_view(switch_lead),
                name='switch_lead'),
            url(
                r'add_lead/(?P<user_id>\d+)/$',
                self.admin_site.admin_view(add_lead),
                name='add_lead'),
            url(
                r'add_lead/$',
                self.admin_site.admin_view(add_lead_user),
                name='add_lead_user'),
            url(
                '^resend_registration_mail/(?P<lead_id>\d+)/$',
                resend_registration_mail,
                name='utenti__resend_registration_mail'),

            # funzioni richiesta cliente
            url(
                r'generazione_codice_conferma_cliente/(?P<lead_id>\d+)/$',
                self.admin_site.admin_view(generazione_codice_conferma_cliente),
                name='generazione_codice_conferma_cliente'),
            url(
                r'set_rac_contratto_invalido/(?P<lead_id>\d+)/(?P<stato>\w+)/$',
                self.admin_site.admin_view(set_rac_contratto_invalido),
                name='set_rac_contratto_invalido'),
            url(
                r'download_lettera_codice_conferma_cliente/(?P<lead_id>\d+)/$',
                self.admin_site.admin_view(download_lettera_codice_conferma_cliente),
                name='download_lettera_codice_conferma_cliente'),

            # cliente
            url(
                r'dashboard_cliente/(?P<user_id>\d+)/$',
                self.admin_site.admin_view(dashboard_cliente),
                name='dashboard_cliente'),
            url(
                r'delete_cliente/(?P<user_id>\d+)/$',
                self.admin_site.admin_view(delete_cliente),
                name='delete_cliente'),
            url(
                r'switch_cliente/(?P<cliente_id>\d+)/$',
                self.admin_site.admin_view(switch_cliente),
                name='switch_cliente'),
            url(
                r'switch_cliente_posizione/(?P<cliente_id>\d+)/$',
                self.admin_site.admin_view(switch_cliente_posizione),
                name='switch_cliente_posizione'),
            url(
                r'add_cliente/(?P<user_id>\d+)/$',
                self.admin_site.admin_view(add_cliente),
                name='add_cliente'),
            url(
                r'add_cliente/$',
                self.admin_site.admin_view(add_cliente_user),
                name='add_cliente_user'),

            # promotore
            url(
                r'dashboard_promotore/(?P<user_id>\d+)/$',
                self.admin_site.admin_view(dashboard_promotore),
                name='dashboard_promotore'),
            url(
                r'delete_promotore/(?P<user_id>\d+)/$',
                self.admin_site.admin_view(delete_promotore),
                name='delete_promotore'),
            url(
                r'switch_promotore/(?P<promotore_id>\d+)/$',
                self.admin_site.admin_view(switch_promotore),
                name='switch_promotore'),
            url(
                r'add_promotore/(?P<user_id>\d+)/$',
                self.admin_site.admin_view(add_promotore),
                name='add_promotore'),
            url(
                r'add_promotore/$',
                self.admin_site.admin_view(add_promotore_user),
                name='add_promotore_user'),

            # rete
            url(
                r'dashboard_utenterete/(?P<user_id>\d+)/$',
                self.admin_site.admin_view(dashboard_utenterete),
                name='dashboard_utenterete'),
            url(
                r'delete_rete/(?P<user_id>\d+)/$',
                self.admin_site.admin_view(delete_rete),
                name='delete_rete'),
            url(
                r'add_rete/(?P<user_id>\d+)/$',
                self.admin_site.admin_view(add_rete),
                name='add_rete'),
            url(
                r'add_rete/$',
                self.admin_site.admin_view(add_rete_user),
                name='add_rete_user'),

            # acomea
            url(
                r'dashboard_acomea/(?P<user_id>\d+)/$',
                self.admin_site.admin_view(dashboard_acomea),
                name='dashboard_acomea'),
            url(
                r'delete_acomea/(?P<user_id>\d+)/$',
                self.admin_site.admin_view(delete_acomea),
                name='delete_acomea'),
            url(
                r'add_acomea/(?P<user_id>\d+)/$',
                self.admin_site.admin_view(add_acomea),
                name='add_acomea'),
            url(
                r'add_acomea/$',
                self.admin_site.admin_view(add_acomea_user),
                name='add_acomea_user'),

            url(
                r'dashboard_messages/$',
                self.admin_site.admin_view(dashboard_messages),
                name='dashboard-messages'),
            url(r'redirect_to_reverse/(?P<slug>\w+)/(?P<user_id>\d+)/$',
                self.admin_site.admin_view(redirect_to_reverse),
                name='redirect-to-reverse'),

            # simulazione
            url(
                r'simulazione_ptf/(?P<user_id>\d+)/$',
                self.admin_site.admin_view(simulazione_ptf),
                name='simulazione_ptf'),
            # offusca dati
            url(
                r'(?P<user_id>\d+)/offusca_dati/$',
                self.admin_site.admin_view(offusca_dati),
                name='offusca_dati'),
        )
        return my_urls + urls

    def get_url_lead(self, obj):
        if User.objects.get(pk=obj.id).get_profile().get_profile(LeadProfile):
            rev = reverse('admin:dashboard_lead', args=(obj.id,))
            rev_del = reverse('admin:delete_lead', args=(obj.id,))
            return "<a href='%s' target='_blank' title='modifica le proprietà lead'><img src='/static/jmb/images/url_icon.png' /></a>&nbsp;" \
                   "<a href='%s' title='elimina lead'><img src='/static/jmb/images/del.gif' /></a>" % (
                rev, rev_del)
        else:
            try:
                p = User.objects.get(pk=obj.id).get_profile()
                rev = reverse('admin:add_lead', args=(obj.id,))
                return "<a href='%s' title='aggiungi un profilo lead'><img src='/static/jmb/images/icon_addlink.gif' /></a>" % (
                    rev,)
            except:
                return ''
    get_url_lead.allow_tags = True
    get_url_lead.short_description = 'lead'

    def get_url_cliente(self, obj):
        if User.objects.get(pk=obj.id).get_profile().get_profile(ClienteProfile):
            rev = reverse('admin:dashboard_cliente', args=(obj.id,))
            rev_del = reverse('admin:delete_cliente', args=(obj.id,))
            return "<a href='%s' target='_blank' title='modifica le proprietà cliente'><img src='/static/jmb/images/url_icon.png' /></a>&nbsp;" \
                   "<a href='%s' title='elimina cliente'><img src='/static/jmb/images/del.gif' /></a>" % (
                rev, rev_del)
        else:
            try:
                # per poter diventare cliente devo prima avere un profilo Lead
                if User.objects.get(pk=obj.id).get_profile().get_profile(LeadProfile):
                    rev = reverse('admin:add_cliente', args=(obj.id,))
                    return "<a href='%s' title='aggiungi un profilo cliente'><img src='/static/jmb/images/icon_addlink.gif' /></a>" % (
                        rev,)
                else:
                    return ''
            except Exception, e:
                return ''
    get_url_cliente.allow_tags = True
    get_url_cliente.short_description = 'cliente'

    def get_url_promotore(self, obj):
        if User.objects.get(pk=obj.id).get_profile().get_profile(PromotoreProfile):
            rev = reverse('admin:dashboard_promotore', args=(obj.id,))
            rev_del = reverse('admin:delete_promotore', args=(obj.id,))
            return "<a href='%s' target='_blank' title='modifica le proprietà promotore'><img src='/static/jmb/images/url_icon.png' /></a>&nbsp;" \
                   "<a href='%s' title='elimina promotore'><img src='/static/jmb/images/del.gif' /></a>" % (
                rev, rev_del)
        else:
            if User.objects.get(pk=obj.id).get_profile():
                rev = reverse('admin:add_promotore', args=(obj.id,))
                return "<a href='%s' title='aggiungi un profilo promotore'><img src='/static/jmb/images/icon_addlink.gif' /></a>" % (
                    rev,)
            else:
                return ''
    get_url_promotore.allow_tags = True
    get_url_promotore.short_description = 'promotore'

    def get_url_rete(self, obj):
        if User.objects.get(pk=obj.id).get_profile().get_profile(UtenteReteProfile):
            rev = reverse('admin:dashboard_utenterete', args=(obj.id,))
            rev_del = reverse('admin:delete_rete', args=(obj.id,))
            return "<a href='%s' target='_blank' title='dashboard utente rete'><img src='/static/jmb/images/url_icon.png' /></a>&nbsp;" \
                   "<a href='%s' title='elimina promotore'><img src='/static/jmb/images/del.gif' /></a>" % (
                rev, rev_del)
        else:
            if User.objects.get(pk=obj.id).get_profile():
                rev = reverse('admin:add_rete', args=(obj.id,))
                return "<a href='%s' title='aggiungi un profilo rete'><img src='/static/jmb/images/icon_addlink.gif' /></a>" % (
                    rev,)
            else:
                return ''
    get_url_rete.allow_tags = True
    get_url_rete.short_description = 'rete'

    def get_url_acomea(self, obj):
        if User.objects.get(pk=obj.id).get_profile().get_profile(AcomeaProfile):
            rev = reverse('admin:dashboard_acomea', args=(obj.id,))
            rev_del = reverse('admin:delete_acomea', args=(obj.id,))
            return "<a href='%s' target='_blank' title='dashboard acomea'><img src='/static/jmb/images/url_icon.png' /></a>&nbsp;" \
                   "<a href='%s' title='elimina acomea'><img src='/static/jmb/images/del.gif' /></a>" % (
                rev, rev_del)
        else:# AcomeaProfile.DoesNotExist:
            if User.objects.get(pk=obj.id).get_profile():
                rev = reverse('admin:add_acomea', args=(obj.id,))
                return "<a href='%s' title='aggiungi un profilo acomea'><img src='/static/jmb/images/icon_addlink.gif' /></a>" % (
                    rev,)
            else:
                return ''
    get_url_acomea.allow_tags = True
    get_url_acomea.short_description = 'acomea'


admin.site.unregister(User)
admin.site.register(User, AcomeAUserAdmin)

admin.site.register(AcomeaRegistrationProfile)
