# -*- coding: utf-8 -*-
import logging
import datetime
import persistent_messages
import sha
import string


from django.conf import settings
from django.contrib import messages
from django.contrib.auth.models import User, Group
from django.contrib.auth.decorators import login_required, user_passes_test
from django.contrib.admin.models import LogEntry, CHANGE
from django.contrib.contenttypes.models import ContentType
from django.core.mail import EmailMultiAlternatives
from django.core.urlresolvers import reverse
from django.db import transaction
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils.encoding import force_unicode
from django.utils.translation import ugettext_lazy as _

from jmb.core.views.generic.create_oo_file import create_file

from acomea.soap.models import AcomeaClientCms
from acomea.utenti.forms import (
    AcomeaLeadEditForm, TestoMessaggioForm, AcomeaLeadAddForm, AddLeadUserForm,
    AddClienteForm, AddClienteUserForm, AddPromotoreForm, AddPromotoreUserForm,
    AddUtenteReteForm, AddReteUserForm, AddAcomeaUserForm, AcomeaCodiceNodoForm
)
from acomea.utenti.models import (
    LeadProfile, ClienteProfile, RichiestaAttivazioneCliente, STATI_RAC, PromotoreProfile,
    Rete, UtenteReteProfile, AcomeaProfile, add_lead_profile_to_user, genera_codice_di_conferma_cliente,
    ExceptionSoapConnection, OneTimePassword
)
from acomea.simulatoreptf.simulazione import Simulazione
from persistent_messages.models import Message
from sccore.profiles.models import SCProfiles


log = logging.getLogger('acomea.utenti')


def genera_one_time_password(request):
    log.debug("STEPINTO: genera_one_time_password")

    sessionid = request.COOKIES['sessionid']
    millitime = "%s%s" % (datetime.datetime.now().second, datetime.datetime.now().microsecond)
    charset = "%s%s" % (millitime, sessionid)
    charlen = len(charset)
    c1 = []
    n = 0
    s = sha.sha(millitime).hexdigest()
    for nibble in s:
        n = (n + string.atoi(nibble, 16)) % charlen
        c1.append(charset[n])
    n = 0
    c2 = []
    for nibble in s:
        n = (n + string.atoi(nibble, 16)) % 40
        c2.append(c1[n])
    output = "".join(c2[-40:])
    log.debug("generato: %s" % output)
    return output


# office per debug
#/usr/bin/soffice --headless --nologo --nofirststartwizard --accept="socket,host=localhost,port=8100;urp;StarOffice.service"
@login_required
def download_lettera_codice_conferma_cliente(request, lead_id):
    log.debug("STEPINTO: download_lettera_codice_conferma_cliente")
    l = LeadProfile.objects.get(pk=lead_id)
    rac = RichiestaAttivazioneCliente.objects.get(leadprofile=l)
    if datetime.date.today().weekday() == 4:
        pd = datetime.date.today() + datetime.timedelta(days=3)
    else:
        pd = datetime.date.today() + datetime.timedelta(days=1)
    ctx = {'get_print_date': "%s/%s/%s" % (pd.day, pd.month, pd.year)}
    return create_file(
        request, model=RichiestaAttivazioneCliente, obj_id=rac.pk,
        file_model='utenti/contratti/lettera_codice_conferma_cliente.odt',
        file_type='pdf', extra_context=ctx
    )


@login_required
#@user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def dashboard_cliente(request, user_id):
    log.debug("STEPINTO: dashboard_cliente")
    u = User.objects.get(pk=user_id)
    c = u.get_profile().get_profile(ClienteProfile)
    rac = u.get_profile().get_profile(LeadProfile).get_rac()

    return render_to_response(
        'utenti/backoffice/dashboard_cliente.html',
        locals(), RequestContext(request)
    )


@login_required
#@user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def dashboard_promotore(request, user_id):
    log.debug("STEPINTO: dashboard_promotore")
    p = User.objects.get(pk=user_id).get_profile().get_profile(PromotoreProfile)
    return render_to_response(
        'utenti/backoffice/dashboard_promotore.html',
        locals(), RequestContext(request)
    )


@login_required
#@user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def dashboard_utenterete(request, user_id):
    log.debug("STEPINTO: dashboard_utenterete")
    r = User.objects.get(pk=user_id).get_profile().get_profile(UtenteReteProfile)
    return render_to_response(
        'utenti/backoffice/dashboard_utenterete.html',
        locals(), RequestContext(request)
    )


@login_required
#@user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def dashboard_acomea(request, user_id):
    log.debug("STEPINTO: dashboard_acomea")
    a = User.objects.get(pk=user_id).get_profile().get_profile(AcomeaProfile)
    return render_to_response(
        'utenti/backoffice/dashboard_acomea.html',
        locals(), RequestContext(request)
    )


@login_required
# @user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def delete_acomea(request, user_id):
    log.debug("STEPINTO: delete_acomea")
    User.objects.get(pk=user_id).get_profile().remove_profile(AcomeaProfile)
    return HttpResponseRedirect(reverse('admin:auth_user_changelist'))


@login_required
#@user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def delete_rete(request, user_id):
    log.debug("STEPINTO: delete_rete")
    User.objects.get(pk=user_id).get_profile().remove_profile(UtenteReteProfile)
    return HttpResponseRedirect(reverse('admin:auth_user_changelist'))


@login_required
#@user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def delete_promotore(request, user_id):
    log.debug("STEPINTO: delete_promotore")
    User.objects.get(pk=user_id).get_profile().remove_profile(PromotoreProfile)
    return HttpResponseRedirect(reverse('admin:auth_user_changelist'))


@login_required
#@user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def delete_cliente(request, user_id):
    log.debug("STEPINTO: delete_cliente")
    u = User.objects.get(pk=user_id)
    u.get_profile().remove_profile(ClienteProfile)
    lead = u.get_profile().get_profile(LeadProfile)
    rac = lead.get_rac()
    rac.stato = '1'
    rac.save()
    return HttpResponseRedirect(reverse('admin:auth_user_changelist'))


@login_required
#@user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def delete_lead(request, user_id):
    log.debug("STEPINTO: delete_lead")
    # la business logic impedisce che esista un Cliente che non sia Lead
    try:
        User.objects.get(pk=user_id).get_profile().remove_profile(ClienteProfile)
    except:
        pass
    User.objects.get(pk=user_id).get_profile().remove_profile(LeadProfile)
    return HttpResponseRedirect(reverse('admin:auth_user_changelist'))


@login_required
# @user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def add_lead(request, user_id):
    log.debug("STEPINTO: add_lead")
    u = User.objects.get(pk=user_id)
    profiles = u.get_profile()
    if request.method == 'POST':
        form = AcomeaLeadAddForm(request.POST)
        if form.is_valid():
            if u.email:
                u.username = u.email
                u.save()
                add_lead_profile_to_user(u, form.cleaned_data)
                p = profiles.get_profile("LeadProfile", "acomea.utenti.models")
                p.abilitato = True
                p.save()
            return HttpResponseRedirect(reverse('admin:auth_user_changelist'))
    else:
        form = AcomeaLeadAddForm()
    return render_to_response(
        'utenti/backoffice/add_lead.html',
        locals(), RequestContext(request)
    )


@login_required
# @user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def add_lead_user(request):
    log.debug("STEPINTO: add_lead_user")
    if request.method == 'POST':
        form = AddLeadUserForm(request.POST)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect(reverse('admin:auth_user_changelist'))
    else:
        form = AddLeadUserForm()
    return render_to_response(
        'utenti/backoffice/add_lead_user.html',
        locals(), RequestContext(request)
    )


@login_required
#@user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def add_cliente(request, user_id):
    log.debug("STEPINTO: add_cliente")
    profiles = User.objects.get(pk=user_id).get_profile()
    if request.method == 'POST':
        form = AddClienteForm(request.POST)
        if form.is_valid():
            lead = profiles.get_profile(LeadProfile)
            rac = lead.get_rac()
            rac.stato = '1'
            rac.codice_fiscale = form.cleaned_data['codice_fiscale']
            try:
                if form.cleaned_data['posizione_desiderata'] == '1':
                    if rac.richiedi_posizione_consultiva(rac.codice_fiscale):
                        rac.codice_conferma_consultivo = genera_codice_di_conferma_cliente(None)
                        rac.stato = '6'
                        profiles.add_profile(
                            "ClienteProfile", "acomea.utenti.models",
                            {
                                'abilitato': True,
                                'dispositivo': False
                            }
                        )
                else:
                    if rac.richiedi_posizione_dispositiva(rac.codice_fiscale):
                        ccc = genera_codice_di_conferma_cliente(None)
                        rac.codice_conferma_consultivo = ccc
                        rac.codice_conferma_dispositivo = ccc
                        rac.stato = '9'
                        profiles.add_profile(
                            "ClienteProfile", "acomea.utenti.models",
                            {
                                'abilitato': True,
                                'dispositivo': True
                            }
                        )
                rac.save()
            except ExceptionSoapConnection:
                bo = User.objects.get(username='backoffice')
                m = persistent_messages.add_message(
                    request, messages.INFO, "errore nel collegamento a soap.",
                    from_user=bo, user=request.user
                )
            return HttpResponseRedirect(reverse('admin:auth_user_changelist'))
    else:
        form = AddClienteForm()
    return render_to_response(
        'utenti/backoffice/add_cliente.html',
        locals(), RequestContext(request)
    )


@login_required
#@user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def add_cliente_user(request):
    log.debug("STEPINTO: add_cliente_user")
    if request.method == 'POST':
        form = AddClienteUserForm(request.POST)
        if form.is_valid():
            l = form.save_lead()
            rac = l.get_rac()
            cf = form.cleaned_data['codice_fiscale']
            pos = form.cleaned_data['posizione_desiderata']
            rac.codice_fiscale = form.cleaned_data['codice_fiscale']
            if form.cleaned_data['posizione_desiderata'] == '1':
                if rac.richiedi_posizione_consultiva(cf):
                    rac.accetta_contratto()
                    rac.approva_posizione()
                    rac.sblocca_posizione(rac.codice_conferma_consultivo)
                else:
                    pass  # TODO messaggio errore
            else:
                if rac.richiedi_posizione_dispositiva(cf):
                    rac.approva_posizione()
                else:
                    pass  # TODO messaggio errore
            return HttpResponseRedirect(reverse('admin:auth_user_changelist'))
    else:
        form = AddClienteUserForm()
    return render_to_response(
        'utenti/backoffice/add_cliente_user.html',
        locals(), RequestContext(request)
    )


@login_required
#@user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def add_promotore(request, user_id):
    log.debug("STEPINTO: add_promotore")
    profiles = User.objects.get(pk=user_id).get_profile()
    if request.method == 'POST':
        form = AddPromotoreForm(request.POST)
        if form.is_valid():
            profiles.add_profile(
                "PromotoreProfile", "acomea.utenti.models", {
                    'codice': form.cleaned_data['codice_promotore']
                }
            )
            return HttpResponseRedirect(reverse('admin:auth_user_changelist'))
    else:
        form = AddPromotoreForm()
    return render_to_response(
        'utenti/backoffice/add_promotore.html',
        locals(), RequestContext(request)
    )


@login_required
#@user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def add_promotore_user(request):
    log.debug("STEPINTO: add_promotore_user")
    if request.method == 'POST':
        form = AddPromotoreUserForm(request.POST)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect(reverse('admin:auth_user_changelist'))
    else:
        form = AddPromotoreUserForm()
    return render_to_response(
        'utenti/backoffice/add_promotore_user.html',
        locals(), RequestContext(request)
    )


@login_required
# @user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def add_rete(request, user_id):
    log.debug("STEPINTO: add_rete")
    profiles = User.objects.get(pk=user_id).get_profile()
    if request.method == 'POST':
        form = AddUtenteReteForm(request.POST)
        if form.is_valid():
            profiles.add_profile(
                "UtenteReteProfile", "acomea.utenti.models",
                {
                    'abilitato': True,
                    'rete': Rete.objects.get(pk=form.cleaned_data['rete'])
                }
            )
            return HttpResponseRedirect(reverse('admin:auth_user_changelist'))
    else:
        form = AddUtenteReteForm()
    return render_to_response(
        'utenti/backoffice/add_utenterete.html',
        locals(), RequestContext(request)
    )


@login_required
# @user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def add_rete_user(request):
    log.debug("STEPINTO: add_rete_user")
    if request.method == 'POST':
        form = AddReteUserForm(request.POST)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect(reverse('admin:auth_user_changelist'))
    else:
        form = AddReteUserForm()
    return render_to_response(
        'utenti/backoffice/add_rete_user.html',
        locals(), RequestContext(request)
    )


@login_required
# @user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def add_acomea(request, user_id):
    log.debug("STEPINTO: add_acomea")
    profiles = User.objects.get(pk=user_id).get_profile()
    profiles.add_profile(AcomeaProfile)
    return HttpResponseRedirect(reverse('admin:auth_user_changelist'))


@login_required
# @user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def add_acomea_user(request):
    log.debug("STEPINTO : add_acomea_user")
    if request.method == 'POST':
        form = AddAcomeaUserForm(request.POST)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect(reverse('admin:auth_user_changelist'))
    else:
        form = AddAcomeaUserForm()
    return render_to_response(
        'utenti/backoffice/add_acomea_user.html',
        locals(), RequestContext(request)
    )


@login_required
#@user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def dashboard_messages(request):
    log.debug("STEPINTO : dashboard_messages")
    if request.method == 'POST':
        form = AcomeaCodiceNodoForm(request.POST)
        if form.is_valid():
            try:
                codice = form.cleaned_data['codice_nodo']
                if form.cleaned_data['profilo_tipo'] == 'privato':
                    utente = ClienteProfile.get_user_from_codice_albero(codice)
                    profiles = SCProfiles.objects.get(user=utente)
                    pc = profiles.get_profile(ClienteProfile)
                    o = OneTimePassword.objects.create(
                        otp=genera_one_time_password(request),
                        sessionid=request.COOKIES['sessionid'],
                        utente_richiedente=utente,
                        codice_richiedente=pc.get_codice_albero())
                    return HttpResponseRedirect("%s/Login.aspx?OTP=%s&SessionID=%s" % (
                        settings.FRAMS_HOST_CLIENT, o.otp, request.COOKIES['sessionid']))
                else:
                    utente = PromotoreProfile.get_user_from_codice_albero(codice)
                    profiles = SCProfiles.objects.get(user=utente)
                    pp = profiles.get_profile(PromotoreProfile)
                    o = OneTimePassword.objects.create(
                        otp=genera_one_time_password(request),
                        sessionid=request.COOKIES['sessionid'],
                        utente_richiedente=utente,
                        codice_richiedente=pp.get_codice_albero())

                    return HttpResponseRedirect("%s/login/?OTP=%s&SessionID=%s" % (
                        settings.FRAMS_HOST_PRIVATE, o.otp, request.COOKIES['sessionid']))
            except:
                pass
    form = AcomeaCodiceNodoForm()
    try:
        bog = Group.objects.get(name='BACKOFFICE')
        messages_todo = Message.objects.filter(
            pertinence_group__exact=bog, read__exact=False
        ).order_by('created')
        messages_done = Message.objects.filter(
            pertinence_group__exact=bog, read__exact=True
        ).order_by('modified')
    except:
        pass
    return render_to_response('utenti/backoffice/dashboard.html', locals(), RequestContext(request))


@login_required
#@user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def dashboard_lead(request, user_id):
    log.debug("STEPINTO : dashboard_lead")
    u = User.objects.get(pk=user_id)
    l = u.get_profile().get_profile(LeadProfile)
    if request.method == 'POST':
        form = AcomeaLeadEditForm(request.POST, lead=l)
        if form.is_valid():
            form.save(lead=l)
        u = User.objects.get(pk=user_id)
        l = u.get_profile().get_profile(LeadProfile)
    else:
        form = AcomeaLeadEditForm(lead=l)
    rac = RichiestaAttivazioneCliente.objects.get(leadprofile=l)
    stato_rac = STATI_RAC[str(rac.stato)]
    if rac.stato != '1':  # se esiste uno stato di richiesta / esistenza del cliente
        try:
            ac = AcomeaClientCms()
        except Exception, e:
            ac = None
            soap_error = True
        try:
            data = ac.get_cliente(l.codice_fiscale)
        except Exception, e:
            data = []
            cf_error = True
        gestioni_collettive, gestioni_individuali = [], []

        for s_data in data:
            try:
                if s_data['TIPO'] == 'Gestioni Collettive':
                    gestioni_collettive = s_data
                else:
                    gestioni_individuali = s_data
            except:
                pass
    return render_to_response('utenti/backoffice/dashboard_lead.html', locals(), RequestContext(request))


@login_required
#@user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def switch_lead(request, lead_id):
    log.debug("STEPINTO : switch_lead")
    l = LeadProfile.objects.get(pk=lead_id)
    if request.method == 'POST':
        form = TestoMessaggioForm(request.POST)
        if form.is_valid():
            l.abilitato = not l.abilitato
            l.save()
            return HttpResponseRedirect(reverse('admin:dashboard_lead', args=[l.get_user().pk]))
    if l.abilitato:
        # se lo sto disabilitando mostro un form per inserire il motivo
        form = TestoMessaggioForm()
        return render_to_response(
            'utenti/backoffice/switch_lead.html',
            locals(), RequestContext(request)
        )
    else:
        l.abilitato = not l.abilitato
        l.save()
        return HttpResponseRedirect(reverse(
            'admin:dashboard_lead', args=[l.get_user().pk]
        ))


@login_required
#@user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def resend_registration_mail(request, lead_id):
    log.debug("STEPINTO: resend_registration_mail")
    l = LeadProfile.objects.get(pk=lead_id)
    from .models import AcomeaRegistrationProfile
    rp = AcomeaRegistrationProfile.objects.get(user=l.get_user())
    from django.contrib.sites.models import Site
    rp.send_activation_email(Site.objects.all()[0])
    return HttpResponseRedirect(reverse('admin:dashboard_lead', args=[l.get_user().pk]))


@login_required
#@user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def switch_cliente(request, cliente_id):
    log.debug("STEPINTO: switch_cliente")
    c = ClienteProfile.objects.get(pk=cliente_id)
    if request.method == 'POST':
        form = TestoMessaggioForm(request.POST)
        if form.is_valid():
            c.abilitato = not c.abilitato
            c.save()
            return HttpResponseRedirect(reverse('admin:dashboard_lead', args=[c.get_user().pk]))
    if c.abilitato:
        # se lo sto disabilitando mostro un form per inserire il motivo
        form = TestoMessaggioForm()
        return render_to_response('utenti/backoffice/switch_cliente.html', locals(), RequestContext(request))
    else:
        c.abilitato = not c.abilitato
        c.save()
        return HttpResponseRedirect(reverse('admin:dashboard_cliente', args=[c.get_user().pk]))


@login_required
#@user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def switch_promotore(request, promotore_id):
    p = PromotoreProfile.objects.get(pk=promotore_id)
    p.abilitato = not p.abilitato
    p.save()
    return HttpResponseRedirect(reverse('admin:dashboard_promotore', args=[p.get_user().pk]))


@login_required
#@user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def switch_cliente_posizione(request, cliente_id):
    log.debug("STEPINTO: switch_cliente_posizione")
    c = ClienteProfile.objects.get(pk=cliente_id)
    if request.method == 'POST':
        form = TestoMessaggioForm(request.POST)
        if form.is_valid():
            c.dispositivo = not c.dispositivo
            c.save()
            return HttpResponseRedirect(reverse('admin:dashboard_cliente', args=[c.get_user().pk]))
    if c.dispositivo:
        # se lo sto disabilitando mostro un form per inserire il motivo
        form = TestoMessaggioForm()
        return render_to_response('utenti/backoffice/switch_cliente.html', locals(), RequestContext(request))
    else:
        # se lo sto abilitando devo anche assicurarmi che esista il codice
        # in posizione dispositiva per coerenza di stato.
        c.dispositivo = not c.dispositivo
        c.save()
        l = c.get_profiles().get_profile(LeadProfile)
        r = l.get_rac()
        r.codice_conferma_dispositivo = r.codice_conferma_consultivo
        r.stato = '9'
        r.save()
        return HttpResponseRedirect(reverse('admin:dashboard_cliente', args=[c.get_user().pk]))


@login_required
def set_rac_contratto_invalido(request, lead_id, stato):
    log.debug("STEPINTO: set_rac_contratto_invalido")
    l = LeadProfile.objects.get(pk=lead_id)
    rac = RichiestaAttivazioneCliente.objects.get(leadprofile=l)
    if request.method == 'POST':
        form = TestoMessaggioForm(request.POST)
        if form.is_valid():
            rac.disapprova_posizione()
            persistent_messages.add_message(
                request, persistent_messages.WARNING,
                "Contratto non accettato per : %s" % form.cleaned_data['testo'], user=l.get_user()
            )
            ctx_dict = {
                'user': l.get_user(),
                'motivo': form.cleaned_data['testo']
            }
            subject = render_to_string('utenti/backoffice/richiestaclienteinvalida_email_subject.txt', ctx_dict)
            subject = ''.join(subject.splitlines())

            text_message = render_to_string('utenti/backoffice/richiestaclienteinvalida_email.txt', ctx_dict)
            html_message = render_to_string('utenti/backoffice/richiestaclienteinvalida_email.html', ctx_dict)
            try:
                msg = EmailMultiAlternatives(
                    subject, text_message,
                    settings.REGISTER_EMAIL_FROM, [l.get_user().email]
                )
                msg.attach_alternative(html_message, "text/html")
                msg.send()
            except Exception, e:  # TODO catchare solo la connection refused
                pass
        return HttpResponseRedirect(reverse('admin:dashboard_lead', args=[l.get_user().pk]))
    else:
        form = TestoMessaggioForm()
        return render_to_response(
            'utenti/backoffice/set_rac_contratto_invalido.html',
            locals(), RequestContext(request)
        )


@login_required
#@user_passes_test(lambda u: u.groups.filter(name='BACKOFFICE').count() == 1)
def generazione_codice_conferma_cliente(request, lead_id):
    log.debug("STEPINTO: generazione_codice_conferma_cliente")
    l = LeadProfile.objects.get(pk=lead_id)
    rac = RichiestaAttivazioneCliente.objects.get(leadprofile=l)
    try:
        rac.approva_posizione()
        ctx_dict = {'user': l.get_user()}
        subject = render_to_string('utenti/backoffice/confermacliente_email_subject.txt', ctx_dict)
        subject = ''.join(subject.splitlines())

        text_message = render_to_string('utenti/backoffice/confermacliente_email.txt', ctx_dict)
        html_message = render_to_string('utenti/backoffice/confermacliente_email.html', ctx_dict)
        try:
            msg = EmailMultiAlternatives(
                subject, text_message,
                settings.REGISTER_EMAIL_FROM, [l.get_user().email]
            )
            msg.attach_alternative(html_message, "text/html")
            msg.send()
        except Exception, e:  # TODO catchare solo la connection refused
            pass
    except:
        raise
    return HttpResponseRedirect(reverse('admin:dashboard_lead', args=[l.get_user().pk]))


def redirect_to_reverse(request, slug, user_id):
    log.debug("STEPINTO: redirect_to_reverse")
    return HttpResponseRedirect(reverse('admin:%s' % slug, args=[user_id]))


def simulazione_ptf(request, user_id):
    user = User.objects.get(pk=user_id)
    simulazione = Simulazione(user)
    extra_context = {'user': user, 'simulazione': simulazione}
    return render_to_response('admin/auth/user/simulazione_ptf.html', extra_context, RequestContext(request))


@transaction.commit_manually
def offusca_dati(request, user_id):
    try:
        # delete is replaced in acomea.utenti#models (search #REPLACE_DELETE_OBFUSCATE)
        user = User.objects.get(pk=user_id)
        user.delete()
        LogEntry.objects.log_action(
            user_id=request.user.pk,
            content_type_id=ContentType.objects.get_for_model(user).pk,
            object_id=user.pk,
            object_repr=force_unicode(user),
            action_flag=CHANGE,
            change_message=_("Offuscato")
        )
        messages.info(request, 'Operazione eseguita con successo')
        transaction.commit()
    except Exception as e:
        messages.error(request, e.message)
        transaction.rollback()

    return HttpResponseRedirect(request.META['HTTP_REFERER'])
