'''
Created on 23/nov/2011

@author: simone cittadini
'''
import sys

from django.conf import settings
from django.contrib.sites.models import RequestSite

from registration import signals
from .forms import AcomeaLeadRegistrationForm
from .models import AcomeaRegistrationProfile


class AcomeaBackend(object):
    def register(self, request, **kwargs):
        username, email, password = kwargs['email'].strip(), kwargs['email'].strip(), kwargs['password1']
        site = RequestSite(request)
        new_user = AcomeaRegistrationProfile.objects.create_inactive_user(
                                    username, email, password, site,
                                    kwargs['nome'], kwargs['cognome'])
        signals.user_registered.send(sender=self.__class__,
                                     user=new_user,
                                     request=request)
        return new_user

    def activate(self, request, activation_key):
        activated = AcomeaRegistrationProfile.objects.activate_user(activation_key)
        if activated:
            signals.user_activated.send(sender=self.__class__,
                                        user=activated,
                                        request=request)
        return activated

    def registration_allowed(self, request):
        """
        Indicate whether account registration is currently permitted,
        based on the value of the setting ``REGISTRATION_OPEN``. This
        is determined as follows:

        * If ``REGISTRATION_OPEN`` is not specified in settings, or is
          set to ``True``, registration is permitted.

        * If ``REGISTRATION_OPEN`` is both specified and set to
          ``False``, registration is not permitted.
        
        """
        return getattr(settings, 'REGISTRATION_OPEN', True)

    def get_form_class(self, request):
        """
        Return the default form class used for user registration.
        
        """
        return AcomeaLeadRegistrationForm

    def post_registration_redirect(self, request, user):
        """
        Return the name of the URL to redirect to after successful
        user registration.
        
        """
        return ('registration_complete', (), {})

    def post_activation_redirect(self, request, user):
        """
        Return the name of the URL to redirect to after successful
        account activation.
        
        """
        return ('registration_activation_complete', (), {})