# -*- coding: utf-8 -*-
from django.db import models
from django.conf import settings
from django.utils.translation import ugettext as _

from jmb.core.db.cmsmodels import StatusModel
from cms.models.pluginmodel import CMSPlugin


class PosizioneIntestazioneSettings(CMSPlugin):
    template_name = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_INTESTAZIONE_TEMPLATES,
        default="utenti/cms_plugins/utenti_intestazione.html"
    )

    class Meta:
        app_label = "utenti"

    def __unicode__(self):
        return self.template_name


class PosizioneDettaglioSettings(CMSPlugin):
    template_name = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_DETTAGLIO_TEMPLATES,
        default="utenti/cms_plugins/utenti_dettaglio.html"
    )

    class Meta:
        app_label = "utenti"

    def __unicode__(self):
        return self.template_name


class PosizioneRiepilogoSettings(CMSPlugin):
    template_name = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_RIEPILOGO_TEMPLATES,
        default="utenti/cms_plugins/utenti_riepilogo.html"
    )

    class Meta:
        app_label = "utenti"

    def __unicode__(self):
        return self.template_name


class PosizioneAcomeASettings(CMSPlugin):
    template_name = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_ACOMEA_TEMPLATES,
        default="utenti/cms_plugins/utenti_acomea.html"
    )

    class Meta:
        app_label = "utenti"

    def __unicode__(self):
        return self.template_name


class PosizioneReteSettings(CMSPlugin):
    template_name = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_RETE_TEMPLATES,
        default="utenti/cms_plugins/utenti_rete.html"
    )

    class Meta:
        app_label = "utenti"

    def __unicode__(self):
        return self.template_name


class PosizionePromotoreSettings(CMSPlugin):
    template_name = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_PROMOTORE_TEMPLATES,
        default="utenti/cms_plugins/utenti_promotore.html"
    )

    class Meta:
        app_label = "utenti"

    def __unicode__(self):
        return self.template_name


class LoginSettings(CMSPlugin):
    slug = models.SlugField(
        max_length=100,
        null=True, blank=True
    )
    classes = models.CharField(
        max_length=100,
        null=True, blank=True
    )
    redirect_to = models.CharField(
        _("Redirect to"),
        max_length=150,
        null=True, blank=True,
        default='la-tua-posizione',
        help_text=_("page slug for redirection after successful login "
                    "(if url resolution fails, redirect will be made on current page)")
    )
    registration_link = models.CharField(
        _("Registration page URL"),
        max_length=150,
        null=True, blank=True,
        help_text=_("relative URL (e.g. '/registration-page/') to registration page "
                    "(accordingly in template must be present a button/link for it)")
    )
    lostpassword_link = models.CharField(
        _("Lost password page URL"),
        max_length=150,
        null=True, blank=True,
        help_text=_("relative URL (e.g. '/password-recovery/') to lost password page "
                    "(accordingly in template must be present a button/link for it)")
    )
    privacy_mgmt_link = models.CharField(
        _("Privacy management page URL"),
        max_length=170,
        null=True, blank=True,
        help_text=_("relative URL (e.g. '/privacy-management/') to privacy management page ")
    )
    template_name = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_LOGIN_TEMPLATES,
        default="utenti/cms_plugins/login.html"
    )

    def __unicode__(self):
        return "Login - %s" % self.id

    class Meta:
        app_label = "utenti"


class RegistrazioneLeadNewSettings(CMSPlugin, StatusModel):
    slug = models.SlugField(
        max_length=100,
        null=True, blank=True
    )
    classes = models.CharField(
        max_length=100,
        null=True, blank=True
    )
    sent_from = models.EmailField(
        _('Indirizzo mittente delle mail'),
        default="sender@example.org",
    )
    form = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_REGISTRATIONNEW_FORM_CLASSES,
        default="acomea.utenti.forms.AcomeaLeadRegistrationNewForm",
    )
    form_template_name = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_REGISTRATIONNEW_FORM_TEMPLATES,
        default="utenti/cms_plugins/registration_new/registration_form.html"
    )
    email_registration_done_template = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_REGISTRATIONNEW_EMAIL_REGISTRATIONDONE_TEMPLATES,
        default="utenti/cms_plugins/registration_new/registration_done_email.html"
    )
    email_activation_done_template = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_REGISTRATIONNEW_EMAIL_ACTIVATIONDONE_TEMPLATES,
        default="utenti/cms_plugins/registration_new/activation_done_email.html"
    )
    # slug pagine registration_done, activation_done
    registration_done_slug = models.SlugField(
        _("Registration done page"),
        help_text=_("Registration done page slug")
    )
    activation_done_slug = models.SlugField(
        _("Activation done page"),
        help_text=_("Activation done page slug")
    )

    def __unicode__(self):
        return "%s %s" % (('On - ' if self.status else 'Off - '), self.form_template_name)

    class Meta:
        app_label = "utenti"


class CheckCodiceFiscaleSettings(CMSPlugin, StatusModel):
    form = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_CHECKFISCALCODE_FORM_CLASSES,
        default="acomea.utenti.forms.CodiceFiscaleForm",
    )
    form_template_name = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_CHECKFISCALCODE_FORM_TEMPLATES,
        default="utenti/cms_plugins/registrazione_new/codice_fiscale_form.html"
    )
    # slug pagine success, unsuccess check cod. fiscale
    success_slug = models.SlugField(
        _("Success page"),
        help_text=_("slug pagina successo codice fiscale")
    )
    unsuccess_slug = models.SlugField(
        _("Unsuccess page"),
        help_text=_("slug pagina insuccesso codice fiscale")
    )

    def __unicode__(self):
        return "%s %s" % (('On - ' if self.status else 'Off - '), self.form_template_name)

    class Meta:
        app_label = "utenti"


class AcomeaClienteConfermaSettings(CMSPlugin, StatusModel):
    form = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_CLIENTECONFERMA_FORM_CLASSES,
        default="acomea.utenti.forms.AcomeaClienteConfermaForm",
    )
    form_template_name = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_CLIENTECONFERMA_FORM_TEMPLATES,
        default="utenti/cms_plugins/registration_new/cliente_conferma_form.html"
    )
    # slug pagine success, unsuccess
    success_slug = models.SlugField(
        _("Success page"),
        help_text=_("slug pagina successo conferma cliente")
    )
    unsuccess_slug = models.SlugField(
        _("Unsuccess page"),
        help_text=_("slug pagina insuccesso conferma cliente")
    )

    def __unicode__(self):
        return "%s %s" % (('On - ' if self.status else 'Off - '), self.form_template_name)

    class Meta:
        app_label = "utenti"


class RichiestaCodiceSbloccoSettings(CMSPlugin, StatusModel):
    form = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_RICHIESTACODICESBLOCCO_FORM_CLASSES,
        default="acomea.utenti.forms.RichiestaCodiceSbloccoForm",
    )
    form_template_name = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_RICHIESTACODICESBLOCCO_FORM_TEMPLATES,
        default="utenti/cms_plugins/registration_new/richiesta_codice_sblocco_form.html"
    )
    # slug pagine success, unsuccess
    success_slug = models.SlugField(
        _("Success page"),
        help_text=_("slug pagina successo richiesta codice sblocco")
    )
    unsuccess_slug = models.SlugField(
        _("Unsuccess page"),
        help_text=_("slug pagina insuccesso richiesta codice sblocco")
    )

    def __unicode__(self):
        return "%s %s" % (('On - ' if self.status else 'Off - '), self.form_template_name)

    class Meta:
        app_label = "utenti"


class RegistrazioneLeadSettings(CMSPlugin):
    slug = models.SlugField(
        max_length=100,
        null=True, blank=True
    )
    classes = models.CharField(
        max_length=100,
        null=True, blank=True
    )
    sent_from = models.EmailField(
        _('Indirizzo mittente delle mail'),
        default="sender@example.org",
    )
    thanks = models.CharField(_('Messaggio di conferma'), max_length=200)

    # FIXME: da eliminare
    form_template_name = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_SUBSCRIPTION_FORM_TEMPLATES,
        default="utenti/cms_plugins/registration_form.html"
    )
    thanks_template_name = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_SUBSCRIPTION_THANKS_TEMPLATES,
        default="utenti/cms_plugins/registration_thanks.html"
    )
    email_template_name = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_SUBSCRIPTION_EMAIL_TEMPLATES,
        default="utenti/cms_plugins/registration_email.html"
    )

    def __unicode__(self):
        return self.form_template_name

    class Meta:
        app_label = "utenti"


class RichiestaAttivazioneClienteSettings(CMSPlugin):
    slug = models.SlugField(
        max_length=100,
        null=True, blank=True
    )
    classes = models.CharField(
        max_length=100,
        null=True, blank=True
    )

    def __unicode__(self):
        return "Richiesta Attivazione Cliente - %s" % self.id

    class Meta:
        app_label = "utenti"


class GestioneClienteSettings(CMSPlugin):
    slug = models.SlugField(
        max_length=100,
        null=True, blank=True
    )
    classes = models.CharField(
        max_length=100,
        null=True, blank=True
    )

    def __unicode__(self):
        return "Gestione Cliente - %s" % self.id

    class Meta:
        app_label = "utenti"


class RecoveryPasswordSettings(CMSPlugin):
    form = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_PASSWORDRECOVERY_FORM_CLASSES,
        default="acomea.utenti.forms.RecoveryPasswordForm",
    )
    sent_from = models.EmailField(
        _('Indirizzo mittente delle mail'),
        default="info@example.org",
    )
    password_reset_pageslug = models.CharField(
        _('Slug della pagina di reset password (deve essere una pagina esistente)'),
        max_length=200,
        default="password-reset",
    )
    done_message = models.CharField(_('Messaggio di conferma'), max_length=200)
    form_template_name = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_PASSWORDRECOVERY_FORM_TEMPLATES,
        default="utenti/cms_plugins/registration/recovery_password_form.html"
    )
    done_template_name = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_PASSWORDRECOVERY_DONE_TEMPLATES,
        default="utenti/cms_plugins/registration/recovery_password_done.html"
    )
    email_template_name = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_PASSWORDRECOVERY_EMAIL_TEMPLATES,
        default="utenti/cms_plugins/registration/recovery_password_email.html"
    )
    def __unicode__(self):
        return "recovery password %s" % self.id

    class Meta:
        app_label = "utenti"


class ResetPasswordSettings(CMSPlugin):
    form_notlogged_in = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_PASSWORDRESET_NOTLOGGEDIN_FORM_CLASSES,
        default="acomea.utenti.forms.ResetPasswordForm",
    )
    form_logged_in = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_PASSWORDRESET_LOGGEDIN_FORM_CLASSES,
        default="acomea.utenti.forms.SetPasswordFormWOldPW",
    )
    done_message = models.CharField(_('Messaggio di conferma'), max_length=200)
    form_template_name = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_PASSWORDRESET_FORM_TEMPLATES,
        default="utenti/cms_plugins/registration/reset_password_form.html"
    )
    done_template_name = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_PASSWORDRESET_DONE_TEMPLATES,
        default="utenti/cms_plugins/registration/reset_password_done.html"
    )
    email_template_name = models.CharField(
        max_length=100,
        choices=settings.UTENTI_CMSPLUGIN_PASSWORDRESET_EMAIL_TEMPLATES,
        default="utenti/cms_plugins/registration/reset_password_email.html"
    )
    redirect_to = models.CharField(
        _("Redirect to"),
        max_length=150,
        null=True, blank=True,
        default='la-tua-posizione',
        help_text=_("(Optional) page slug for fallback redirection")
    )

    def __unicode__(self):
        return "reset password %s" % self.id

    class Meta:
        app_label = "utenti"