#! -*- coding: utf-8 -*-
'''
Created on 01/set/2011

@author: simone cittadini
'''
import re
from django.contrib.auth.models import User
from django.contrib.auth.forms import PasswordResetForm, SetPasswordForm, PasswordChangeForm
from django.contrib.auth.tokens import default_token_generator
from django import forms
from django.utils.translation import ugettext_lazy as _
from django.core.mail import EmailMessage
from django.core.mail import EmailMultiAlternatives
from django.template.loader import get_template, render_to_string
from django.template import Context
from django.contrib.sites.models import get_current_site
from django.utils.http import int_to_base36
from django.conf import settings

from cms import models as cm_models
from captcha.fields import ReCaptchaField as CaptchaField

#from acomea.soap.models import AcomeaClientCms
#from .models import LeadProfile
from web.utils.generic import get_acomea_host, get_acomea_tcp_protocol

class LoginForm(forms.Form):
    username = forms.CharField(max_length=100)
    password = forms.CharField(max_length=100, widget=forms.PasswordInput())

# I put this on all required fields, because it's easier to pick up
# on them with CSS or JavaScript if they have a class of "required"
# in the HTML. Your mileage may vary. If/when Django ticket #3515
# lands in trunk, this will no longer be necessary.
attrs_dict = {'class': 'required'}

class AcomeaLeadEditForm(forms.Form):
    # admin form
    datanasc = forms.DateField(required=False)
    res_indirizzo = forms.CharField()
    res_citta = forms.CharField()
    res_prov = forms.CharField()
    res_cap = forms.CharField()
    telefono = forms.CharField(required=False)

    def __init__(self, *args , **kwargs):
        lead = kwargs.pop('lead', None)
        super(AcomeaLeadEditForm, self).__init__(*args, **kwargs)
        if lead:
            self.fields['datanasc'].initial = lead.datanasc
            self.fields['res_indirizzo'].initial = lead.res_indirizzo
            self.fields['res_citta'].initial = lead.res_citta
            self.fields['res_prov'].initial = lead.res_prov
            self.fields['res_cap'].initial = lead.res_cap
            self.fields['telefono'].initial = lead.telefono

    def save(self, *args, **kwargs):
        lead = kwargs.pop('lead', None)
        if lead:
            lead.data_di_nascita = self.cleaned_data['datanasc']
            lead.indirizzo = self.cleaned_data['res_indirizzo']
            lead.citta = self.cleaned_data['res_citta']
            lead.provincia = self.cleaned_data['res_prov']
            lead.cap = self.cleaned_data['res_cap']
            lead.telefono = self.cleaned_data['telefono']
            lead.save()

class AcomeaLeadAddForm(forms.Form):
    # admin form
    nickname = forms.CharField(label="* Nickname")
    data_di_nascita = forms.DateField(required=False,
        label="Data di nascita (gg-mm-aaaa)")
    telefono = forms.CharField(required=False)
    indirizzo = forms.CharField(required=False)
    citta = forms.CharField(required=False, label="Città")
    cap = forms.CharField(required=False)
    provincia = forms.CharField(required=False)
    hobby_uno = forms.CharField(required=False)
    hobby_due = forms.CharField(required=False)
    hobby_tre = forms.CharField(required=False)

    mercato_di_interesse = forms.MultipleChoiceField(
        widget=forms.CheckboxSelectMultiple,
        choices=(
            ('Azionario', 'Azionario'),
            ('Obbligazionario', 'Obbligazionario'),
            ('Monetario', 'Monetario')
        ),
        label="Mercato di interesse",
        required=False)
    professionale = forms.BooleanField(required=False,
        label="Investitore professionale")
    #auth_marketing = forms.BooleanField(initial=True, required=False)

    def clean_email(self):
        try:
            user = User.objects.get(username__iexact=self.cleaned_data['email'])
        except User.DoesNotExist:
            return self.cleaned_data['email']
        raise forms.ValidationError(_("A user with that username already exists."))

    def clean_nickname(self):
        try:
            l = LeadProfile.objects.get(nickname=self.cleaned_data['nickname'])
        except LeadProfile.DoesNotExist:
            return self.cleaned_data['nickname']
        raise forms.ValidationError("Un utente con questo nickname esiste già.")


GROUPS_CHOICES = (
    (6, 'Clienti',),
    (5, 'Privati',),
    (7, 'Professionali',),
)
class AcomeaLeadRegistrationForm(forms.Form):
    email = forms.EmailField(
        widget=forms.TextInput(attrs=dict(attrs_dict,
        maxlength=50)), # was 30)),
        label="* Indirizzo email"
    )
    nickname = forms.CharField(label="* Nickname")
    password1 = forms.CharField(
        widget=forms.PasswordInput(attrs=attrs_dict,
        render_value=False),
        label=_("* Password")
    )
    password2 = forms.CharField(
        widget=forms.PasswordInput(attrs=attrs_dict,
        render_value=False),
        label=_("* Ripeti password")
    )
    nome = forms.CharField(label="* Nome")
    cognome = forms.CharField(label="* Cognome")
    professionale = forms.BooleanField(required=False, label="Investitore professionale")
    iscrizione_alla_newsletter = forms.BooleanField(initial=True, required=False)
    gruppo_newsletter = forms.ChoiceField(label=_('*Gruppo'), choices=GROUPS_CHOICES)
    captcha = CaptchaField(label="* Digita le lettere")
    auth_marketing = forms.BooleanField(initial=True, required=False, label="Autorizzazione")

    def clean_email(self):
        try:
            user = User.objects.get(username__iexact=self.cleaned_data['email'])
        except User.DoesNotExist:
            return self.cleaned_data['email']
        raise forms.ValidationError(_("A user with that username already exists."))

    def clean_nickname(self):
        try:
            l = LeadProfile.objects.get(nickname=self.cleaned_data['nickname'])
        except LeadProfile.DoesNotExist:
            return self.cleaned_data['nickname']
        raise forms.ValidationError("Un utente con questo nickname esiste già.")

    def clean_password1(self):
        password1 = self.cleaned_data.get('password1')

        if len(password1) < 8:
            raise forms.ValidationError("La password deve essere di almeno 8 caratteri.")

        first_isalpha = password1[0].isalpha()
        if all(c.isalpha() == first_isalpha for c in password1):
            raise forms.ValidationError("La password deve contenere almeno una lettera e un numero o carattere di punteggiatura.")
        return password1

    def clean(self):
        if 'password1' in self.cleaned_data and 'password2' in self.cleaned_data:
            if self.cleaned_data['password1'] != self.cleaned_data['password2']:
                raise forms.ValidationError(_("The two password fields didn't match."))
        return self.cleaned_data

class AcomeaLeadRegistrationNewForm(forms.Form):
    AUTH_CHOICES = (
        (1, "Accetto",),
        (0, "Non accetto",),
    )
    nome = forms.CharField(label="* Nome")
    cognome = forms.CharField(label="* Cognome")
    email = forms.EmailField(
        widget=forms.TextInput(attrs=dict(attrs_dict, maxlength=50)), # was 30)),
        label="* Email"
    )
    nickname = forms.CharField(label="* Username")
    password1 = forms.CharField(
        widget=forms.PasswordInput(attrs=attrs_dict, render_value=False),
        label=_("* Password")
    )
    password2 = forms.CharField(
        widget=forms.PasswordInput(attrs=attrs_dict, render_value=False),
        label=_("* Ripeti password")
    )
    professionale = forms.BooleanField(required=False, label="Investitore professionale")
    iscrizione_alla_newsletter = forms.BooleanField(initial=True, required=False)
    captcha = CaptchaField(label="* Digita le lettere")
    auth_trattamento_dati = forms.ChoiceField(initial=True, choices=AUTH_CHOICES, widget=forms.RadioSelect(), required=False)
    auth_societa_terze = forms.ChoiceField(initial=True, choices=AUTH_CHOICES, widget=forms.RadioSelect(), required=False)
    auth_analisi = forms.ChoiceField(initial=True, choices=AUTH_CHOICES, widget=forms.RadioSelect(), required=False)

    def clean_email(self):
        try:
            user = User.objects.get(username__iexact=self.cleaned_data['email'])
        except User.DoesNotExist:
            return self.cleaned_data['email']
        raise forms.ValidationError(_("A user with that username already exists."))

    def clean_nickname(self):
        try:
            l = LeadProfile.objects.get(nickname=self.cleaned_data['nickname'])
        except LeadProfile.DoesNotExist:
            return self.cleaned_data['nickname']
        raise forms.ValidationError("Un utente con questo nickname esiste già.")

    def clean_password1(self):
        password1 = self.cleaned_data.get('password1')

        if len(password1) < 8:
            raise forms.ValidationError("La password deve essere di almeno 8 caratteri.")

        first_isalpha = password1[0].isalpha()
        if all(c.isalpha() == first_isalpha for c in password1):
            raise forms.ValidationError("La password deve contenere almeno una lettera e un numero o carattere di punteggiatura.")
        return password1

    def clean(self):
        if 'password1' in self.cleaned_data and 'password2' in self.cleaned_data:
            if self.cleaned_data['password1'] != self.cleaned_data['password2']:
                raise forms.ValidationError(_("The two password fields didn't match."))
        return self.cleaned_data

class CodiceFiscaleForm(forms.Form):
    codice_fiscale = forms.CharField(label="Codice Fiscale")

class RichiestaCodiceSbloccoForm(forms.Form):
    pass


# class AcomeaClientCmsRichiestaForm(forms.Form):
#     posizione_desiderata = forms.ChoiceField(choices=(
#         ("1", "consultiva"),
#         ("2", "dispositiva")
#     ))
#     codice_fiscale = forms.CharField(max_length=16)
#     indirizzo = forms.CharField()
#     citta = forms.CharField()
#     cap = forms.CharField(max_length=5)
#     provincia = forms.CharField(max_length=2)

class AcomeaClienteConfermaForm(forms.Form):
    codice_conferma_uno = forms.CharField(widget=forms.TextInput(attrs={'size':'4', 'maxlength':'4s', 'class':'input-mini'}))
    codice_conferma_due = forms.CharField(widget=forms.TextInput(attrs={'size':'4', 'maxlength':'4s', 'class':'input-mini'}))
    codice_conferma_tre = forms.CharField(widget=forms.TextInput(attrs={'size':'4', 'maxlength':'4s', 'class':'input-mini'}))
    codice_conferma_quattro = forms.CharField(widget=forms.TextInput(attrs={'size':'4', 'maxlength':'4s', 'class':'input-mini'}))

class AcomeaCodiceNodoForm(forms.Form):
    codice_nodo = forms.CharField(label=u"Inserisci Codice Nodo")
    profilo_tipo = forms.ChoiceField(choices=(('privato', 'privato'),
                                              ('professionale', 'professionale')))

class AcomeaIndirizzoSpedizioneForm(forms.Form):
    indirizzo = forms.CharField(required=False)
    citta = forms.CharField(required=False)
    cap = forms.CharField(required=False)

class AddLeadForm(AcomeaLeadRegistrationForm):
    pass

class AddLeadUserForm(forms.Form):
    email = forms.EmailField(widget=forms.TextInput(attrs=dict(attrs_dict,
        maxlength=75)),
        label=_("Email address"))
    password1 = forms.CharField(widget=forms.PasswordInput(attrs=attrs_dict,
        render_value=False),
        label=_("Password"))
    password2 = forms.CharField(widget=forms.PasswordInput(attrs=attrs_dict,
        render_value=False),
        label=_("Password (again)"))
    nome = forms.CharField()
    cognome = forms.CharField()
    data_di_nascita = forms.DateField(required=False, label="Data di nascita (gg-mm-aaaa)")
    telefono = forms.CharField(required=False)
    indirizzo = forms.CharField(required=False)
    citta = forms.CharField(required=False)
    cap = forms.CharField(required=False)
    hobby_uno = forms.CharField(required=False)
    hobby_due = forms.CharField(required=False)
    hobby_tre = forms.CharField(required=False)
    mercato_di_interesse = forms.CharField(required=False)
    iscrizione_alla_newsletter = forms.BooleanField(required=False)

    def clean_email(self):
        try:
            user = User.objects.get(username__iexact=self.cleaned_data['email'])
        except User.DoesNotExist:
            return self.cleaned_data['email']
        raise forms.ValidationError(_("A user with that username already exists."))

    def clean_password1(self):
        password1 = self.cleaned_data.get('password1')

        if len(password1) < 8:
            raise forms.ValidationError("La password deve essere di almeno 8 caratteri.")

        first_isalpha = password1[0].isalpha()
        if all(c.isalpha() == first_isalpha for c in password1):
            raise forms.ValidationError("La password deve contenere almeno una lettera e un numero o carattere di punteggiatura.")
        return password1

    def clean(self):
        if 'password1' in self.cleaned_data and 'password2' in self.cleaned_data:
            if self.cleaned_data['password1'] != self.cleaned_data['password2']:
                raise forms.ValidationError(_("The two password fields didn't match."))
        return self.cleaned_data

    def save(self, *args, **kwargs):
        u = User.objects.create_user(
            self.cleaned_data['email'],
            self.cleaned_data['email'],
            self.cleaned_data['password1']
        )
        u.first_name = self.cleaned_data['nome']
        u.last_name = self.cleaned_data['cognome']
        u.save()
        profiles = u.get_profile()
        profiles.creato_da_backoffice = True
        profiles.save()
        l = profiles.add_profile("LeadProfile", "acomea.utenti.models",
                {'abilitato' : True})
        from .models import RichiestaAttivazioneCliente
        rac = RichiestaAttivazioneCliente.objects.create(leadprofile=l)

class AddClienteForm(forms.Form):
    codice_fiscale = forms.CharField()
    posizione_desiderata = forms.ChoiceField(choices=(("1", "consultiva"),
                                                      ("2", "dispositiva")))

class AddClienteUserForm(forms.Form):
    email = forms.EmailField(widget=forms.TextInput(attrs=dict(attrs_dict,
        maxlength=75)),
        label=_("Email address"))
    password1 = forms.CharField(widget=forms.PasswordInput(attrs=attrs_dict,
        render_value=False),
        label=_("Password"))
    password2 = forms.CharField(widget=forms.PasswordInput(attrs=attrs_dict,
        render_value=False),
        label=_("Password (again)"))
    nome = forms.CharField()
    cognome = forms.CharField()
    posizione_desiderata = forms.ChoiceField(choices=(("1", "consultiva"),
                                                      ("2", "dispositiva")))
    codice_fiscale = forms.CharField()
    data_di_nascita = forms.DateField(required=False)
    telefono = forms.CharField(required=False)
    indirizzo = forms.CharField()
    citta = forms.CharField()
    cap = forms.CharField()
    hobby_uno = forms.CharField(required=False)
    hobby_due = forms.CharField(required=False)
    hobby_tre = forms.CharField(required=False)
    mercato_di_interesse = forms.CharField(required=False)
    iscrizione_alla_newsletter = forms.BooleanField(required=False)

    def clean_email(self):
        try:
            user = User.objects.get(username__iexact=self.cleaned_data['email'])
        except User.DoesNotExist:
            return self.cleaned_data['email']
        raise forms.ValidationError(_("A user with that username already exists."))

    def clean_password1(self):
        password1 = self.cleaned_data.get('password1')

        if len(password1) < 8:
            raise forms.ValidationError("La password deve essere di almeno 8 caratteri.")

        first_isalpha = password1[0].isalpha()
        if all(c.isalpha() == first_isalpha for c in password1):
            raise forms.ValidationError("La password deve contenere almeno una lettera e un numero o carattere di punteggiatura.")
        return password1

    def clean(self):
        if 'password1' in self.cleaned_data and 'password2' in self.cleaned_data:
            if self.cleaned_data['password1'] != self.cleaned_data['password2']:
                raise forms.ValidationError(_("The two password fields didn't match."))
        ac = AcomeaClientCms()
        result = ac.client.service.CheckCliente(cf=self.cleaned_data['codice_fiscale'])
        if result.CheckResult != 'OK':
            raise forms.ValidationError(_("Il codice fiscale non corrisponde a un cliente."))
        if result.CheckResult == 'OK' and not result.CanOperate and self.cleaned_data['posizione_desiderata'] == '2':
            raise forms.ValidationError(_("Il cliente non può essere dispositivo."))
        return self.cleaned_data

    def save_lead(self, *args, **kwargs):
        u = User.objects.create_user(
            self.cleaned_data['email'],
            self.cleaned_data['email'],
            self.cleaned_data['password1']
        )
        u.first_name = self.cleaned_data['nome']
        u.last_name = self.cleaned_data['cognome']
        u.save()
        profiles = u.get_profile()
        profiles.creato_da_backoffice = True
        profiles.save()
        l = profiles.add_profile("LeadProfile", "acomea.utenti.models",
                {'abilitato' : True})
        from .models import RichiestaAttivazioneCliente
        rac = RichiestaAttivazioneCliente.objects.create(leadprofile=l)
        return l

class AddPromotoreForm(forms.Form):
    codice_promotore = forms.CharField()

class AddPromotoreUserForm(forms.Form):
    email = forms.EmailField(widget=forms.TextInput(attrs=dict(attrs_dict,
        maxlength=75)),
        label=_("Email address"))
    password1 = forms.CharField(widget=forms.PasswordInput(attrs=attrs_dict,
        render_value=False),
        label=_("Password"))
    password2 = forms.CharField(widget=forms.PasswordInput(attrs=attrs_dict,
        render_value=False),
        label=_("Password (again)"))
    nome = forms.CharField()
    cognome = forms.CharField()
    codice_promotore = forms.CharField()

    def clean_email(self):
        try:
            user = User.objects.get(username__iexact=self.cleaned_data['email'])
        except User.DoesNotExist:
            return self.cleaned_data['email']
        raise forms.ValidationError(_("A user with that username already exists."))

    def clean_password1(self):
        password1 = self.cleaned_data.get('password1')

        if len(password1) < 8:
            raise forms.ValidationError("La password deve essere di almeno 8 caratteri.")

        first_isalpha = password1[0].isalpha()
        if all(c.isalpha() == first_isalpha for c in password1):
            raise forms.ValidationError("La password deve contenere almeno una lettera e un numero o carattere di punteggiatura.")
        return password1

    def clean(self):
        if 'password1' in self.cleaned_data and 'password2' in self.cleaned_data:
            if self.cleaned_data['password1'] != self.cleaned_data['password2']:
                raise forms.ValidationError(_("The two password fields didn't match."))
        return self.cleaned_data

    def save(self, *args, **kwargs):
        u = User.objects.create_user(self.cleaned_data['email'],
            self.cleaned_data['email'],
            self.cleaned_data['password1'])
        u.first_name = self.cleaned_data['nome']
        u.last_name = self.cleaned_data['cognome']
        u.save()
        profiles = u.get_profile()
        profiles.creato_da_backoffice = True
        profiles.save()
        l = profiles.add_profile("LeadProfile", "acomea.utenti.models",
                {'abilitato' : True})
        from .models import RichiestaAttivazioneCliente
        RichiestaAttivazioneCliente.objects.create(leadprofile=l)
        u = profiles.add_profile("PromotoreProfile", "acomea.utenti.models",
                {'abilitato' : True,
                 'codice' : self.cleaned_data['codice_promotore']})

class AddUtenteReteForm(forms.Form):
    rete = forms.ChoiceField()

    def __init__(self, *args, **kwargs):
        super(AddUtenteReteForm, self).__init__(*args, **kwargs)
        from .models import Rete
        self.fields['rete'].choices = ((r.pk, r.nome) for r in Rete.objects.all())

class AddReteUserForm(forms.Form):
    email = forms.EmailField(widget=forms.TextInput(attrs=dict(attrs_dict,
        maxlength=75)),
        label=_("Email address"))
    password1 = forms.CharField(widget=forms.PasswordInput(attrs=attrs_dict,
        render_value=False),
        label=_("Password"))
    password2 = forms.CharField(widget=forms.PasswordInput(attrs=attrs_dict,
        render_value=False),
        label=_("Password (again)"))
    nome = forms.CharField()
    cognome = forms.CharField()
    rete = forms.ChoiceField()

    def __init__(self, *args, **kwargs):
        super(AddReteUserForm, self).__init__(*args, **kwargs)
        from acomea.utenti.models import Rete
        self.fields['rete'].choices = ((r.pk, r.nome) for r in Rete.objects.all())

    def clean_email(self):
        try:
            user = User.objects.get(username__iexact=self.cleaned_data['email'])
        except User.DoesNotExist:
            return self.cleaned_data['email']
        raise forms.ValidationError(_("A user with that username already exists."))

    def clean_password1(self):
        password1 = self.cleaned_data.get('password1')

        if len(password1) < 8:
            raise forms.ValidationError("La password deve essere di almeno 8 caratteri.")

        first_isalpha = password1[0].isalpha()
        if all(c.isalpha() == first_isalpha for c in password1):
            raise forms.ValidationError("La password deve contenere almeno una lettera e un numero o carattere di punteggiatura.")
        return password1

    def clean(self):
        if 'password1' in self.cleaned_data and 'password2' in self.cleaned_data:
            if self.cleaned_data['password1'] != self.cleaned_data['password2']:
                raise forms.ValidationError(_("The two password fields didn't match."))
        return self.cleaned_data

    def save(self, *args, **kwargs):
        u = User.objects.create_user(
            self.cleaned_data['email'],
            self.cleaned_data['email'],
            self.cleaned_data['password1']
        )
        u.first_name = self.cleaned_data['nome']
        u.last_name = self.cleaned_data['cognome']
        u.save()
        profiles = u.get_profile()
        profiles.creato_da_backoffice = True
        profiles.save()
        l = profiles.add_profile("LeadProfile", "acomea.utenti.models",
                {'abilitato' : True})
        from .models import RichiestaAttivazioneCliente
        rac = RichiestaAttivazioneCliente.objects.create(leadprofile=l)
        from .models import Rete
        u = profiles.add_profile("UtenteReteProfile", "acomea.utenti.models",
                {'abilitato' : True,
                 'rete' : Rete.objects.get(pk=self.cleaned_data['rete'])})

class AddAcomeaUserForm(forms.Form):
    email = forms.EmailField(widget=forms.TextInput(attrs=dict(attrs_dict,
        maxlength=75)),
        label=_("Email address"))
    password1 = forms.CharField(widget=forms.PasswordInput(attrs=attrs_dict,
        render_value=False),
        label=_("Password"))
    password2 = forms.CharField(widget=forms.PasswordInput(attrs=attrs_dict,
        render_value=False),
        label=_("Password (again)"))
    nome = forms.CharField()
    cognome = forms.CharField()

    def clean_email(self):
        try:
            user = User.objects.get(username__iexact=self.cleaned_data['email'])
        except User.DoesNotExist:
            return self.cleaned_data['email']
        raise forms.ValidationError(_("A user with that username already exists."))

    def clean_password1(self):
        password1 = self.cleaned_data.get('password1')

        if len(password1) < 8:
            raise forms.ValidationError("La password deve essere di almeno 8 caratteri.")

        first_isalpha = password1[0].isalpha()
        if all(c.isalpha() == first_isalpha for c in password1):
            raise forms.ValidationError("La password deve contenere almeno una lettera e un numero o carattere di punteggiatura.")
        return password1

    def clean(self):
        if 'password1' in self.cleaned_data and 'password2' in self.cleaned_data:
            if self.cleaned_data['password1'] != self.cleaned_data['password2']:
                raise forms.ValidationError(_("The two password fields didn't match."))
        return self.cleaned_data

    def save(self, *args, **kwargs):
        u = User.objects.create_user(
            self.cleaned_data['email'],
            self.cleaned_data['email'],
            self.cleaned_data['password1']
        )
        u.first_name = self.cleaned_data['nome']
        u.last_name = self.cleaned_data['cognome']
        u.save()
        profiles = u.get_profile()
        profiles.creato_da_backoffice = True
        profiles.save()
        l = profiles.add_profile("LeadProfile", "acomea.utenti.models",
                {'abilitato' : True})
        from .models import RichiestaAttivazioneCliente
        rac = RichiestaAttivazioneCliente.objects.create(leadprofile=l)
        a = profiles.add_profile("AcomeaProfile", "acomea.utenti.models",
                {'abilitato' : True})

class TestoMessaggioForm(forms.Form):
    testo = forms.CharField(widget=forms.Textarea)

class RichiestaAttivazioneForm(forms.Form):
    # posizione_desiderata = forms.ChoiceField(choices=(
    #     ("1", "consultiva"),
    #     ("2", "dispositiva"))
    # )
    codice_fiscale = forms.CharField(max_length=16)
    indirizzo = forms.CharField()
    citta = forms.CharField()
    cap = forms.CharField(max_length=5)
    provincia = forms.CharField(max_length=2)


class RecoveryPasswordForm(PasswordResetForm):
    email = forms.EmailField(
        label=_("E-mail"),
        max_length=150,
        widget=forms.TextInput(attrs={
            'class': 'form-control',
            'placeholder': 'e-mail'
        })
    )

    def save(self, cms_model_instance, request, page_relative_url=None):
        domain_override=None
        use_https=False
        token_generator=default_token_generator
        subject_template_name='registration/password_reset_subject.txt'
        for user in self.users_cache:
            if not domain_override:
                current_site = get_current_site(request)
                site_name = current_site.name
                domain = current_site.domain
            else:
                site_name = domain = domain_override
            if not page_relative_url:
                page_relative_url = request.get_full_path()
            #protocol = "https" if request.is_secure() else "http"
            protocol = get_acomea_tcp_protocol(request)
            ctx_dict = Context({
                'email': user.email,
                'domain': domain,
                'site_name': site_name,
                'uid': int_to_base36(user.id),
                'user': user,
                'token': token_generator.make_token(user),
                'protocol': protocol,
                'page_relative_url':page_relative_url,
                #'host':request.get_host(),
                'host':get_acomea_host(),
                #'static': protocol+"://"+request.get_host()+settings.STATIC_URL,
                'static': protocol+"://"+get_acomea_host()+settings.STATIC_URL,
                #'homepage': protocol+"://"+request.get_host()
                'homepage': protocol+"://"+get_acomea_host()

                })
            html_template = get_template(cms_model_instance.email_template_name)
            text_template = cms_model_instance.email_template_name.split('.')[0]+'.txt'
            subject = "Richiesta recupero password %s " % ctx_dict['site_name']
            from_email=cms_model_instance.sent_from
            to=[user.email]
            try:
                # debug setting: to define in local settings: send the recovery link to specified
                # address instead of real user address
                to=[settings.ACOMEA_UTENTI_LOCAL_RECOVERYPW_EMAIL_SENT_TO]
            except AttributeError:
                pass

            text_message = render_to_string(text_template, ctx_dict)
            html_message = html_template.render(ctx_dict)
            msg = EmailMultiAlternatives(subject, text_message, from_email, to)
            msg.attach_alternative(html_message, "text/html")
            msg.send()



class ResetPasswordForm(SetPasswordForm):
    """
    A form that lets a user change his/her password
    """

    def __init__(self, user, *args, **kwargs):
        super(ResetPasswordForm, self).__init__(user, *args, **kwargs)
        self.user = user
        self.fields['new_password1'].widget.attrs = {'class': 'form-control', 'placeholder': 'nuova password'}
        self.fields['new_password2'].widget.attrs = {'class': 'form-control', 'placeholder': 'conferma nuova password'}


    def clean_new_password1(self):
        score = check_password_strenght(self.cleaned_data.get('new_password1'))[0]
        if score > 1:
            return self.cleaned_data['new_password1']
        raise forms.ValidationError(_("""
            La tua Password e' troppo semplice o breve. Allungala o combina
            lettere maiuscole e minuscole, numeri e simboli.
        """))

    def clean(self):
        self.error_messages['notequal_to_firstname'] = _("La password non puo' essere uguale al tuo Nome")
        self.error_messages['notequal_to_lastname'] = _("La password non puo' essere uguale al tuo Cognome")
        self.error_messages['notequal_to_email'] = _("La password non puo' essere uguale alla tua Email")
        self.error_messages['notderived_from_email'] = _("La password non puo' derivare da parti dell'Email")
        self.error_messages['password_mismatch'] = _("Le password inserite non coincidono")

        new_password1 = self.cleaned_data.get('new_password1')
        new_password2 = self.cleaned_data.get('new_password2')
        first_name = self.user.first_name
        last_name = self.user.last_name
        email = self.user.email
        if email:
            account = email.split('@')[0]

        msg = None
        if new_password1 and first_name and new_password1.lower() == first_name.lower():
            raise forms.ValidationError(self.error_messages['notequal_to_firstname'])
        if new_password1 and last_name and new_password1.lower() == last_name.lower():
            raise forms.ValidationError(self.error_messages['notequal_to_lastname'])
            #msg = u"La password non puo' essere uguale al tuo Cognome"
        if new_password1 and email and new_password1.lower() == email.lower():
            raise forms.ValidationError(self.error_messages['notequal_to_email'])
            #msg = u"La password non puo' essere uguale alla tua Email"
        if new_password1 and account and new_password1.lower() == account.lower():
            raise forms.ValidationError(self.error_messages['notderived_from_email'])
        if new_password1 and new_password2 and new_password1 != new_password2:
                raise forms.ValidationError(self.error_messages['password_mismatch'])
        #msg = u"La password non puo' derivare da parti dell'Email"
        # if msg:
        #     self._errors["new_password1"] = self.error_class([_(msg)])

        # if new_password1 and new_password2 and new_password1 != new_password2:
        #     msg = u"The two password fields didn't match."
        #     self._errors["new_password2"] = self.error_class([_(msg)])
        #     del self.cleaned_data["new_password1"]
        #     del self.cleaned_data["new_password2"]
        return self.cleaned_data


class SetPasswordFormWOldPW(PasswordChangeForm):
    """
    A form that lets a user change his/her password by entering
    their old password.

    """
    def __init__(self, user, *args, **kwargs):
        super(SetPasswordFormWOldPW, self).__init__(user, *args, **kwargs)
        self.user = user
        self.fields['new_password1'].widget.attrs = {'class': 'form-control', 'placeholder': 'nuova password'}
        self.fields['new_password2'].widget.attrs = {'class': 'form-control', 'placeholder': 'conferma nuova password'}
        self.fields['old_password'].widget.attrs = {'class': 'form-control', 'placeholder': 'password attuale'}

        # self.fields['new_password1'] = forms.CharField(label=_("New password"),
        #                                                widget=forms.PasswordInput(attrs={'class':'span8'}))
        # self.fields['new_password2'] = forms.CharField(label=_("New password confirmation"),
        #                                                widget=forms.PasswordInput(attrs={'class':'span8'}))
        # self.fields['old_password'] = forms.CharField(label=_("Old password"),
        #                                               widget=forms.PasswordInput(attrs={'class':'span8'}))
    # old_password = forms.CharField(label=_("Old password"), widget=forms.PasswordInput)
    # def clean_old_password(self):
    #     """
    #     Validates that the old_password field is correct.
    #     """
    #     old_password = self.cleaned_data["old_password"]
    #     if not self.user.check_password(old_password):
    #         raise forms.ValidationError(_("""
    #             Your old password was entered incorrectly. Please enter it again.
    #         """))
    #     return old_password


#######################
#    helper methods   #
#######################

def check_password_strenght(password):
    strength = ['Blank','Very Weak','Weak','Medium','Strong','Very Strong']
    score = 0

    if len(password) < 1:
        return score, strength[0]
    if len(password) < 4:
        return score, strength[1]

    if len(password) >=8:
        score = score + 1
    if len(password) >=11:
        score = score + 1

    if re.search('\d+',password):
        score = score + 1
    if re.search('[a-z]',password) and re.search('[A-Z]',password):
        score = score + 1
    if re.search('.[!,@,#,$,%,^,&,*,?,_,~,-,£,(,)]',password):
        score = score + 1
    return score, strength[score]
