# -*- coding: utf-8 -*-

from django.shortcuts import render_to_response
from django.template import RequestContext
import locale
import logging
import datetime
import sha
import string
from django.core.mail import EmailMultiAlternatives

from django.http import HttpResponseRedirect, Http404, HttpResponseNotFound, HttpResponseServerError, HttpResponse
from django.contrib.auth import login, logout
from django.utils.http import urlquote
from django.contrib.auth.decorators import login_required
from django.conf import settings
from django.utils.translation import ugettext as _
from django.contrib import messages

from registration.backends import get_backend

from jmb.core.utils.ooutils import  OOUtils
from jmb.core.utils.functions import get_template_abspath

from acomea.soap.models import AcomeaClientRiservataCms
from acomea.sso.utils.sso_client import SSOClient
from acomea.utenti.models import AcomeaProfiles, LeadProfile, ClienteProfile, OneTimePassword, PromotoreProfile, UtenteReteProfile, AcomeaProfile


log = logging.getLogger("acomea.utenti")


def login_view(request, user, url_to_redirect=None):
    login(request, user)

    if request.GET:
        if "url" in request.GET:
            return HttpResponseRedirect(urlquote(request.GET["url"]))

    # Controllo startswith
    for path_url in settings.UTENTI_LOGIN_VIEW_REDIRECT_MYACOMEA["startswith"]:
        if request.path.startswith(path_url):
            return HttpResponseRedirect("/my_acomea")

    # Controllo uguaglianza
    if request.path in settings.UTENTI_LOGIN_VIEW_REDIRECT_MYACOMEA["equals"]:
        url = "/my_acomea"
    else:
        url = '/' if not url_to_redirect else url_to_redirect
        messages.success(request, _("benvenuto %s" % request.user.first_name))

    return HttpResponseRedirect(url)


def logout_view(request, url_to_redirect=None):
    first_name = request.user.first_name if not request.user.is_anonymous() else None

    logout(request)

    url = '/' if not url_to_redirect else url_to_redirect
    if request.GET:
        if "url" in request.GET:
            url = urlquote(request.GET["url"])
            if not request.GET["url"].startswith("/my_acomea") and first_name:
                messages.success(request, _("arrivederci %s" % first_name))
    return HttpResponseRedirect(url)


def activate(request, activation_key, **kwargs):
    log.debug("STEPINTO : activate")
    if request.user.is_authenticated():
        raise Http404()

    backend = get_backend('acomea.utenti.backend.AcomeaBackend')
    account = backend.activate(request, activation_key)

    if account:
        p = AcomeaProfiles.objects.get(user=account)
        lp = p.get_profile(LeadProfile)
        lp.abilitato = True
        lp.save()
        return HttpResponseRedirect('/la-tua-posizione/') # pagina "ok, loggati qui"
    return HttpResponseRedirect('/la-tua-posizione/') # pagina codice errato o sei già confermato


def genera_one_time_password(request):
    log.debug("STEPINTO : genera_one_time_password")

    sessionid = request.COOKIES['sessionid']
    millitime = "%s%s" % (datetime.datetime.now().second, datetime.datetime.now().microsecond)
    charset = "%s%s" % (millitime, sessionid)
    charlen = len(charset)
    c1 = []
    n = 0
    s = sha.sha(millitime).hexdigest()
    for nibble in s:
        n = (n + string.atoi(nibble,16)) % charlen
        c1.append(charset[n])
    n = 0
    c2 = []
    for nibble in s:
        n = (n + string.atoi(nibble,16)) % 40
        c2.append(c1[n])
    output = "".join(c2[-40:])
    log.debug("generato : %s" % output)
    return output


@login_required
def frams_posizione_cliente(request, codice=None):
    log.debug("STEPINTO : frams_posizione_cliente")
    if codice == None:
        utente = request.user
        profiles = request.user.get_profile()
        pc = profiles.get_profile(ClienteProfile)
    else:
        utente = ClienteProfile.get_user_from_codice_albero(codice)
        profiles = utente.get_profile()
        pc = profiles.get_profile(ClienteProfile)
    while True:
        try:
            o = OneTimePassword.objects.create(
                otp=genera_one_time_password(request),
                sessionid=request.COOKIES['sessionid'],
                utente_richiedente=utente,
                codice_richiedente=pc.get_codice_albero())
            log.debug("url_verso_frams (cliente): %s/Login.aspx?OTP=%s&SessionID=%s" % (settings.UTENTI_FRAMS_HOST_CLIENT, o.otp, request.COOKIES['sessionid']))
            return HttpResponseRedirect("%s/Login.aspx?OTP=%s&SessionID=%s" % (settings.UTENTI_FRAMS_HOST_CLIENT, o.otp, request.COOKIES['sessionid']))
        except Exception, e:
            frams_notification_recipients = getattr(settings, 'FRAMS_NOTIFICATION_RECIPIENTS', None)
            if frams_notification_recipients:
                msg = EmailMultiAlternatives("eccezione in generazione otp",
                                             "frams_posizione_cliente : eccezione in generazione otp: %s" % str(e),
                                             "ict@acomea.it", frams_notification_recipients)
#                msg.send()


@login_required
def frams_posizione_promotore(request, codice=None):
    log.debug("STEPINTO : frams_posizione_promotore")
    profiles = request.user.get_profile()
    pp = profiles.get_profile(PromotoreProfile)
    while True:
        try:
            o = OneTimePassword.objects.create(
                otp=genera_one_time_password(request),
                sessionid=request.COOKIES['sessionid'],
                utente_richiedente=request.user,
                codice_richiedente=pp.get_codice_albero())
            log.debug("url_verso_frams (promotore): %s/Login.aspx?OTP=%s&SessionID=%s" % (settings.UTENTI_FRAMS_HOST_PRIVATE, o.otp, request.COOKIES['sessionid']))
            return HttpResponseRedirect("%s/login/?OTP=%s&SessionID=%s" % (settings.UTENTI_FRAMS_HOST_PRIVATE, o.otp, request.COOKIES['sessionid']))
        except Exception, e:
            frams_notification_recipients = getattr(settings, 'FRAMS_NOTIFICATION_RECIPIENTS', None)
            if frams_notification_recipients:
                msg = EmailMultiAlternatives("eccezione in generazione otp",
                                             "frams_posizione_promotore : eccezione in generazione otp: %s" % str(e),
                                             "ict@acomea.it", frams_notification_recipients)
#                msg.send()


@login_required
def frams_posizione_rete(request, codice=None):
    log.debug("STEPINTO : frams_posizione_rete")
    profiles = request.user.get_profile()
    pp = profiles.get_profile(UtenteReteProfile)
    while True:
        try:
            o = OneTimePassword.objects.create(
                otp=genera_one_time_password(request),
                sessionid=request.COOKIES['sessionid'],
                utente_richiedente=request.user,
                codice_richiedente=pp.get_codice_albero())
            log.debug("url_verso_frams (rete): %s/Login.aspx?OTP=%s&SessionID=%s" % (settings.UTENTI_FRAMS_HOST_PRIVATE, o.otp, request.COOKIES['sessionid']))
            return HttpResponseRedirect("%s/login/?OTP=%s&SessionID=%s" % (settings.UTENTI_FRAMS_HOST_PRIVATE, o.otp, request.COOKIES['sessionid']))
        except Exception, e:
            frams_notification_recipients = settings.UTENTI_FRAMS_NOTIFICATION_RECIPIENTS
            if frams_notification_recipients:
                msg = EmailMultiAlternatives("eccezione in generazione otp",
                                             "frams_posizione_rete : eccezione in generazione otp: %s" % str(e),
                                             "ict@acomea.it", frams_notification_recipients)
#                msg.send()


@login_required
def frams_posizione_acomea(request):
    log.debug("STEPINTO : frams_posizione_acomea")
    profiles = request.user.get_profile()
    pp = profiles.get_profile(AcomeaProfile)
    while True:
        try:
            o = OneTimePassword.objects.create(otp=genera_one_time_password(request),
                                               sessionid=request.COOKIES['sessionid'],
                                               utente_richiedente=request.user,
                                               codice_richiedente=pp.get_codice_albero())
            log.debug("url_verso_frams (acomea): %s/Login.aspx?OTP=%s&SessionID=%s" % (settings.UTENTI_FRAMS_HOST_PRIVATE, o.otp, request.COOKIES['sessionid']))
            return HttpResponseRedirect("%s/login/?OTP=%s&SessionID=%s" % (settings.UTENTI_FRAMS_HOST_PRIVATE, o.otp, request.COOKIES['sessionid']))
        except Exception, e:
            frams_notification_recipients = settings.UTENTI_FRAMS_NOTIFICATION_RECIPIENTS
            if frams_notification_recipients:
                msg = EmailMultiAlternatives("eccezione in generazione otp",
                                             "frams_posizione_acomea : eccezione in generazione otp: %s" % str(e),
                                             "ict@acomea.it", frams_notification_recipients)
#                msg.send()


def get_codice_albero(request, otp=None, sessionid=None):
    log.debug("STEPINTO : get_codice_albero - utenti")

    try:
        if request.META.has_key('HTTP_X_FORWARDED_FOR'):
            source_ip = request.META['HTTP_X_FORWARDED_FOR']
            log.debug("source IP is %s (forwarded)" % source_ip)
        else:
            source_ip = request.META['REMOTE_ADDR']
            log.debug("source IP is %s (remote)" % source_ip)
        source_net = source_ip[0:source_ip.rindex('.')]
        log.debug("source_net : %s" % source_net)
        range_ips = settings.UTENTI_IPRANGES_GET_CODICIALBERO[source_net]
        log.debug("range_ips : %s" % str(range_ips))
        source_host = int(source_ip[source_ip.rindex('.')+1:])
        log.debug("source_host : %s" % source_host)
        frams_notification_recipients = settings.UTENTI_FRAMS_NOTIFICATION_RECIPIENTS
    except Exception, e:
        log.debug("get_codice_albero, exception : %s" % str(e))

    if source_host >= range_ips[0] and source_host <= range_ips[1]:
        if request.method == 'GET':
            otp = request.GET['OTP']
            sessionid = request.GET['SessionID']
        now = datetime.datetime.now()
        try:
            p = OneTimePassword.objects.get(otp=otp, sessionid=sessionid, usata=False)
            if ( now - p.timestamp ) > datetime.timedelta(minutes=settings.UTENTI_FRAMS_TIMEOUT_SECONDS*60):
                log.debug("get_codice_albero, denied for timeout %s / %s" % (sessionid, otp))
                return HttpResponseNotFound(otp)
        except OneTimePassword.DoesNotExist:
            log.debug("get_codice_albero, denied for unexisting session / OTP : %s / %s" % (sessionid, otp))
            if frams_notification_recipients:
                msg = EmailMultiAlternatives(
                    "get_codice_albero, denied",
                    "get_codice_albero, denied, otp does not exist: sid: %s, otp: %s, ip: %s" % (sessionid, otp, source_ip),
                    "ict@acomea.it", frams_notification_recipients)
                msg.send()
            return HttpResponseNotFound(otp)
        except Exception, e:
            log.debug("get_codice_albero, exception %s" % str(e))
            if frams_notification_recipients:
                msg = EmailMultiAlternatives(
                    "get_codice_albero, exception", "get_codice_albero, exception: %s" % str(e),
                    "ict@acomea.it", frams_notification_recipients)
                msg.send()
            return HttpResponseServerError(otp)
        p.usata = True
        p.save()
        log.debug("get_codice_albero, ritorno : %s" % p.codice_richiedente)
        return HttpResponse(p.codice_richiedente)
    else:
        log.debug("get_codice_albero, denied for source IP : %s" % source_ip)
        if frams_notification_recipients:
            msg = EmailMultiAlternatives(
                "get_codice_albero, denied",
                "get_codice_albero, denied, not permitted source ip: %s" % source_ip,
                "ict@acomea.it", frams_notification_recipients)
            msg.send()
        return HttpResponseNotFound(otp)


def _get_riepilogo(acr, prodotto, partecipazione, codfiscale):
    prodotto = prodotto.upper()
    partecipazione = partecipazione.upper()
    riepilogo = acr.get_riepilogo(prodotto, partecipazione, codfiscale)
    output = {}
    locale.setlocale(locale.LC_ALL, 'it_IT.utf8')
    output['valore_patrimonio'] = locale.format("%.2f", riepilogo.ValATT, grouping=True, monetary=True)
    output['saldo_sottoscrizioni'] = locale.format("%.2f", riepilogo.SaldoSR, grouping=True, monetary=True)
    output['utile_perdita'] = locale.format("%.2f", riepilogo.ProfitLoss, grouping=True, monetary=True)
    output['data_valori'] = str(riepilogo.DataPosizione)[0:10]
    return output


def _get_dettaglio_as_table(acr, prodotto, partecipazione, codfiscale):
    prodotto = prodotto.upper()
    partecipazione = partecipazione.upper()
    dettaglio = acr.get_dettaglio(prodotto, partecipazione, codfiscale)
    rapporti = []
    for _, array_sr in dettaglio:
        linee_rapporto = []
        first = True
        row_span = 0
        for l in array_sr:
            row_span += 1
            linea_rapporto = {}
            if first:
                linea_rapporto['rapporto'] =  l.Rapporto
                linea_rapporto['contratto'] = l.Contratto
            else:
                linea_rapporto['rapporto'] = None
                linea_rapporto['contratto'] = None
            linea_rapporto['prodotti'] = l.DescrProdotto
            linea_rapporto['valori_patrimoniali'] = l.ValATT
            linea_rapporto['saldo'] = l.SaldoSR
            linea_rapporto['utili'] = l.ProfitLoss
            linee_rapporto.append(linea_rapporto)
            first = False
        rapporti.append((linee_rapporto, row_span))
    return rapporti


@login_required
def download_sintesi_posizione_pdf(request):
    prodotto = request.GET['prodotto']
    partecipazione = request.GET['partecipazione']
    file_model = 'utenti/doc/sintesi_posizione.odt'
    file_name = 'sintesi_posizione'
    o = OOUtils()
    path_modello = get_template_abspath(file_model)

    extra_context = {}
    try:
        acr = AcomeaClientRiservataCms()
        righe_dettaglio = acr.get_dettaglio(
            prodotto, partecipazione, request.user.get_profile().get_lead().codice_fiscale
        )
        extra_context = {
            'righe_dettaglio': righe_dettaglio
        }

    except Exception, e:
        pass

    return o.save_pdf(request, path_modello, file_name, extra_context=extra_context)


@login_required
def print_sintesi_posizione(request, *args, **kwargs):
    template_name = kwargs.pop('template_name', 'utenti/doc/print.html')
    prodotto = request.GET['prodotto']
    partecipazione = request.GET['partecipazione']

    try:
        acr = AcomeaClientRiservataCms()
        riepilogo = acr.get_riepilogo(
            prodotto, partecipazione,
            request.user.get_profile().get_lead().codice_fiscale
        )
        righe_dettaglio = acr.get_dettaglio(
            prodotto, partecipazione,
            request.user.get_profile().get_lead().codice_fiscale
        )

    except Exception, e:
        pass

    return render_to_response(template_name, locals(), RequestContext(request))
